' Demonstrate the routines for I2C using the device's MSSP1 peripheral using I2C
' Write bytes to a 24LC256 I2C eeprom, then read them back and display them on the serial terminal
'
' Written by Les Johnson for the Proton BASIC Compiler
'
    Device = 18F25K22                           ' Select a device with an I2C peripheral
    Declare Xtal = 64

    Declare Hserial_Baud = 9600                 ' Set USART1 baud rate to 9600
'
' Define the pins used for the device's MSSP1 peripheral
'
$define Hbus1_SCL_Pin PORTC.3
$define Hbus1_SDA_Pin PORTC.4

    Include "Hbus1.inc"                         ' Load the I2C MSSP1 routines into the program
'
' Create some variables for the demo
'
    Dim wAddress As Word                        ' 16-bit address of eeprom
    Dim ByteIn As Byte                          ' The byte received
    Dim Byteout As Byte                         ' The byte transmitted

'-----------------------------------------------------------------------------------------------
' The main program starts here
' Write bytes to a 24LC256 I2C eeprom, then read them back and display them on the serial terminal
'
Main:
'
' Write to the eeprom
'
    For wAddress = 0 To 9                               ' Create a 10 element loop
        Byteout = wAddress + 100                        ' Load the value to write to eeprom 1
        Hbus1_Start()                                   ' Send a Start condition
        Hbus1_Set_SlaveAddress(%01010000, Hbus1_Write)  ' Send the Device + Slave address + Write bit
        Hbus1_Write_Byte(wAddress.HighByte)             ' Send the High Byte of the address
        Hbus1_Write_Byte(wAddress.LowByte)              ' Send the Low Byte of the address
        Hbus1_Write_Byte(Byteout)                       ' Send the value of Byteout to the eeprom
        Hbus1_Stop()                                    ' Send a Stop condition
        DelayMS 5
    Next
'
' Read from the eeprom
'
    HRSOutLn "From eeprom 1:"
    For wAddress = 0 To 9                               ' Create a 10 element loop
        Hbus1_Start()                                   ' Send a Start condition
        Hbus1_Set_SlaveAddress(%01010000, Hbus1_Write)  ' Send the Device + Slave address + Write bit
        Hbus1_Write_Byte(wAddress.HighByte)             ' Send the High Byte of the address
        Hbus1_Write_Byte(wAddress.LowByte)              ' Send the Low Byte of the address

        Hbus1_ReStart()                                 ' Send a ReStart condition
        Hbus1_Set_SlaveAddress(%01010000, Hbus1_Read)   ' Send the Device + Slave address + Read bit
        Hbus1_Read_Byte(ByteIn)                         ' Read the byte from the eeprom
        Hbus1_Nack()                                    ' Send a Nack condition
        Hbus1_Stop()                                    ' Send a Stop condition
        HRSOutLn "Address ", Dec wAddress, " is holding ", Dec ByteIn ' Display what was read on the serial teminal
        DelayMS 50                                      ' Delay so we can see things happen
    Next
    Stop

'-----------------------------------------------------------------------------------------------
' Setup for 64MHz operation using a 16MHz crystal
'
Config_Start
    FOSC = HSHP           ' HS oscillator (high power > 16 MHz)
    PLLCFG = On           ' Oscillator multiplied by 4
    PRICLKEN = On         ' Primary clock enabled
    FCMEN = Off           ' Fail-Safe Clock Monitor disabled
    IESO = Off            ' Internal/External Oscillator Switchover mode disabled
    PWRTEN = On           ' Power up timer enabled
    BOREN = SBORDIS       ' Brown-out Reset enabled in hardware only (SBOREN is disabled)
    BORV = 190            ' Brown Out Reset Voltage set to 1.90 V nominal
    WDTEN = Off           ' Watch dog timer is always disabled. SWDTEN has no effect.
    WDTPS = 128           ' Watchdog Timer Postscale 1:128
    CCP2MX = PORTC1       ' CCP2 input/output is multiplexed with RC1
    PBADEN = Off          ' PORTB<5:0> pins are configured as digital I/O on Reset
    CCP3MX = PORTB5       ' P3A/CCP3 input/output is multiplexed with RB5
    HFOFST = On           ' HFINTOSC output and ready status are not delayed by the oscillator stable status
    T3CMX = PORTC0        ' Timer3 Clock Input (T3CKI) is on RC0
    P2BMX = PORTB5        ' ECCP2 B (P2B) is on RB5
    MCLRE = EXTMCLR       ' MCLR pin enabled, RE3 input pin disabled
    STVREN = Off          ' Stack full/underflow will not cause Reset
    LVP = Off             ' Single-Supply ICSP disabled
    XINST = Off           ' Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
    Debug = Off           ' Disabled
    Cp0 = Off             ' Block 0 (000800-001FFFh) not code-protected
    CP1 = Off             ' Block 1 (002000-003FFFh) not code-protected
    CP2 = Off             ' Block 2 (004000-005FFFh) not code-protected
    CP3 = Off             ' Block 3 (006000-007FFFh) not code-protected
    CPB = Off             ' Boot block (000000-0007FFh) not code-protected
    CPD = Off             ' Data EEPROM not code-protected
    WRT0 = Off            ' Block 0 (000800-001FFFh) not write-protected
    WRT1 = Off            ' Block 1 (002000-003FFFh) not write-protected
    WRT2 = Off            ' Block 2 (004000-005FFFh) not write-protected
    WRT3 = Off            ' Block 3 (006000-007FFFh) not write-protected
    WRTC = Off            ' Configuration registers (300000-3000FFh) not write-protected
    WRTB = Off            ' Boot Block (000000-0007FFh) not write-protected
    WRTD = Off            ' Data EEPROM not write-protected
    EBTR0 = Off           ' Block 0 (000800-001FFFh) not protected from table reads executed in other blocks
    EBTR1 = Off           ' Block 1 (002000-003FFFh) not protected from table reads executed in other blocks
    EBTR2 = Off           ' Block 2 (004000-005FFFh) not protected from table reads executed in other blocks
    EBTR3 = Off           ' Block 3 (006000-007FFFh) not protected from table reads executed in other blocks
    EBTRB = Off           ' Boot Block (000000-0007FFh) not protected from table reads executed in other blocks
Config_End
